/****************************************************************************
    Timer1.c
    --------
    bungsprojekt 16-Bit Timer/Counter.
    Zeitmessung ber 2 Eingangssignale (Lichtschranken o..).
    Die beiden Signalgeber werden in einem Abstand von 20 cm montiert.
    An PORTB, Pin 0 wird eine Kontroll-LED angeschlossen, welche anzeigt, dass
    eine Messung luft.

    Autor:    Christian Schifferle
    Datum:    Dezember 2002
    Software: AVR-GCC
    needed
****************************************************************************/

#include <io.h>	         // Wir brauchen Zugriff auf die I/O's des Controllers
#include "UartPrintF.h"

#define F_CPU           10000000      /* 10Mhz */
#define UART_BAUD_RATE      9600      /* 9600 baud */

// Berechnung des Inhalts des UBBR-Registers
#define UART_BAUD_SELECT (F_CPU/(UART_BAUD_RATE*16l)-1)

// Datentypdefinitionen
typedef unsigned char	BYTE;
typedef unsigned short	WORD;



int main (void)
/****************************************************************************
    Hier ist der Programmeinstiegspunkt.
*/
{
    register BYTE nStatus=0;    // Status der Messung
                                // 0 - Es luft keine Messung
                                // 1 - Messvorgang gestartet
    register BYTE tifr=0;       // Hilfsvariable zum Speichern des TIFR
    register BYTE nCycles=0;    // Anzahl Durchlufe des Zhlers
    WORD     nTicks;
    int      nSpeed;

    // UART initialisieren
    outp ((1 << TXEN), UCR);
    // Baudrate einstellen
    outp ((BYTE) UART_BAUD_SELECT, UBRR);          

    // Pull-Up-Widerstnde Pin 2 und 6 aktivieren
    sbi (PORTD, PIND2);
    sbi (PORTD, PIND6);

    // An PORTB, Pin 0 schliessen wir eine Kontroll-LED gegen Masse an.
    outp (0xff, DDRB);

    // Eingangsfrequenzen des Timers bei 10000000 CPU-Takt
    // +-----------+--------------+---------------+-----------+
    // | Vorteiler |     Freq.    | Durchlaufzeit | Taktzeit  |
    // +-----------+--------------+---------------+-----------+
    // |     1     | 10000000     |     0.0065536 | 0.0000001 |
    // |     8     |  1250000     |     0.0524288 | 0.0000008 |
    // |    64     |   156250     |     0.4194304 | 0.0000064 |
    // |   256     |    39062.5   |     1.6777216 | 0.0000256 |
    // |  1024     |     9765.625 |     6.7108864 | 0.0001024 |
    // +-----------+--------------+---------------+-----------+
    // 10000000 / 1024 = 9765.625 Hz

    // Und jetzt in die Hauptschleife
    for (;;) {
        // Je nach Status der Messung
        if (nStatus == 0) {
            // Triggersignal vorhanden ?
            if (bit_is_clear (PIND, PIND2)) {
                nCycles = 0;
                // Timer auf 0 setzen und aktivieren
                __outw (0, TCNT1L);
                // Timer-Parameter einstellen, ICP fallende Flanke, Vorteiler 8
                outp ((1<<CS11), TCCR1B);
                nStatus = 1;
                outp (0x01, PORTB);   // Kontroll-LED ein
            }
        }
        else {
            // ICP-Signal erhalten ?
            // Eigentlich wrde man diese Anwendung normalerweise mit
            // einem Interrupt lsen, aber dazu kommen wir spter.
            tifr = inp (TIFR);
            if (tifr & (1<<ICF1)) {
                // Timer stoppen
                outp (0, TCCR1B);
                // Wert auslesen
                nTicks = __inw (ICR1L);
                /* Geschwindigkeit berechnen
                   und ausgeben, dabei sollten keine Fliesskommazahlen
                   verwendet werden.
                   v = S / T;
                   T = (nCycles * 65536 + nTicks) / 1250000
                   S = 1 / 5
                   dann wird v zu
                   v = 1/5 * (1250000 / (nCycles * 65536 + nTicks))
                     = 250000 / (nCycles * 65536 + nTicks)
                   und nun noch in km/h
                   v = (250000 * 3600) / (nCycles * 65536 + nTicks) * 1000
                     = (250 * 3600) / (nCycles * 65536 + nTicks)
                     = 900000 / (nCycles * 65536 + nTicks)
                */
                nSpeed = (int) (900000L / (nCycles * 65536L + (long) nTicks));
                UartPrintF ("%5d km/h\r", nSpeed);
                sbi (TIFR, ICF1);   // ICF-Flag rcksetzen
                nStatus = 0;
                outp (0x00, PORTB);   // Kontroll-LED aus
            }
            // berlauf testen
            else if (tifr & (1<<TOV1)) {
                nStatus = 2;
                nCycles++;
                if (nCycles > 12) {
                    // Abbruch, da Geschwindigkeit zu langsam zum Messen ist
                    // Timer stoppen
                    outp (0, TCCR1B);
                    nStatus = 0;
                    outp (0x00, PORTB);   // Kontroll-LED aus
                    UartPrintF ("EEEEE     \r");
                }
                sbi (TIFR, TOV1);   // TOV1-Flag rcksetzen
            }
        }
    }

} /* end of main */
